@@ -11,7 +11,7 @@ from typing import Optional, List, Any, Dict, ClassVar
 
 from mashumaro import DataClassDictMixin, pass_through
 
-from TikTokLive.types import User, Gift, Emote, TreasureBoxData, ExtraRankData, LinkUser, BattleArmy, RankData
+from TikTokLive.types import User, Gift, Emote, TreasureBoxData, ExtraRankData, LinkUser, BattleArmy, TopViewer, ChatImage
 from TikTokLive.types.utilities import LiveEvent, alias
 
 
@@ -42,8 +42,8 @@ class AbstractEvent(DataClassDictMixin):
         if hasattr(self, "user") and isinstance(self.user, User):
             self.user.avatar._client = client
             for badge in self.user.badges:
-                for image_badge in badge.image_badges:
-                    image_badge._client = client
+                if badge.image:
+                    badge.image._client = client
 
 
 @LiveEvent("connect")
@@ -79,7 +79,7 @@ class LikeEvent(AbstractEvent):
     """Total number of likes sent"""
 
     display_type: Optional[str] = None
-    """Internal type"""
+    """Internal display type"""
 
     label: Optional[str] = None
     """Internal TikTok label"""
@@ -101,6 +101,9 @@ class JoinEvent(AbstractEvent):
     label: Optional[str] = None
     """Label for event in live chat"""
 
+    total_viewers: Optional[int] = None
+    """Total number of viewers in the stream"""
+
     action_id: Optional[int] = None
     """Internal action ID from TikTok"""
 
@@ -112,7 +115,7 @@ class JoinEvent(AbstractEvent):
         :return: Returns True if they joined through a share link
 
         """
-        return self.type == "pm_mt_join_message_other_viewer"
+        return self.display_type == "pm_mt_join_message_other_viewer"
 
 
 @LiveEvent("follow")
@@ -131,6 +134,9 @@ class FollowEvent(AbstractEvent):
     label: Optional[str] = None
     """Internal TikTok label"""
 
+    total_followers: Optional[int] = None
+    """Total number of creator followers"""
+
 
 @LiveEvent("share")
 class ShareEvent(AbstractEvent):
@@ -172,16 +178,19 @@ class MoreShareEvent(ShareEvent):
             return None
 
 
-@LiveEvent("viewer_count")
-class ViewerCountEvent(AbstractEvent):
+@LiveEvent("viewer_update")
+class ViewerUpdateEvent(AbstractEvent):
     """
-    Event that fires when the viewer count for the livestream updates
+    Event that fires when the viewer count & top viewers for the livestream updates
     
     """
 
     viewer_count: Optional[int] = None
     """The number of people viewing the stream currently"""
 
+    top_viewers: List[TopViewer] = field(default_factory=list)
+    """Top 10-20 viewers in the livestream by coins given"""
+
 
 @LiveEvent("comment")
 class CommentEvent(AbstractEvent):
@@ -196,6 +205,15 @@ class CommentEvent(AbstractEvent):
     comment: Optional[str] = None
     """The UTF-8 text comment that was sent"""
 
+    mentions: List[User] = field(default_factory=list)
+    """List of people the comment mentions"""
+
+    images: List[ChatImage] = field(default_factory=list)
+    """List of chat images sent in a comment"""
+
+    language: Optional[str] = None
+    """Language the commenter uses"""
+
 
 @LiveEvent("live_end")
 class LiveEndEvent(AbstractEvent):
@@ -218,6 +236,16 @@ class GiftEvent(AbstractEvent):
     gift: Optional[Gift] = None
     """Object containing gift data"""
 
+    @classmethod
+    def __pre_deserialize__(cls, d: Dict[Any, Any]) -> Dict[Any, Any]:
+        """
+        De-flatten the gift event (too much in the primary payload)
+
+        """
+
+        d["gift"] = d  # Move 'er up a bit
+        return d
+    
     def _forward_client(self, client: TikTokLiveClient):
         """
         Forward the client to events where it is required
@@ -232,16 +260,6 @@ class GiftEvent(AbstractEvent):
             if self.gift.detailed:
                 self.gift.detailed.icon._client = client
 
-    @classmethod
-    def __pre_deserialize__(cls, d: Dict[Any, Any]) -> Dict[Any, Any]:
-        """
-        De-flatten the gift event (too much in the primary payload)
-
-        """
-
-        d["gift"] = d  # Move 'er up a bit
-        return d
-
 
 @LiveEvent("question")
 class QuestionEvent(AbstractEvent):
@@ -332,7 +350,15 @@ class WeeklyRankingEvent(AbstractEvent):
 
         """
 
-        return d.get('data', dict()).get('rankings', dict())  # Do a little flattening
+        ranks: Dict[str, Any] = d.get('data', dict()).get('rankings', dict())  # Do a little flattening
+
+        # Try to flatten rank data & convert to int
+        try:
+            ranks['rank'] = int(ranks.get('data', dict()).get('rank', None))
+        except:
+            pass
+
+        return ranks
 
     type: Optional[str] = None
     """Unknown"""
@@ -340,20 +366,19 @@ class WeeklyRankingEvent(AbstractEvent):
     label: Optional[str] = None
     """Internal TikTok Label"""
 
-    extra: Optional[ExtraRankData] = None
+    extra: Optional[ExtraRankData] = field(default_factory=ExtraRankData)
     """Extra data relating to the UI, presumably"""
 
-    data: Optional[RankData] = None
-    """The actual rank of the user"""
+    rank: Optional[int] = None
+    """The number for the user's TikTok ranking. If the rank is "None", the user is not in the top 99."""
 
     @property
-    def rank(self) -> Optional[int]:
+    def top_99(self) -> bool:
         """
-        The number for the user's TikTok ranking
-
+        Whether the user is in the top 99 of creators
         """
 
-        return self.data.rank
+        return self.rank is not None
 
 
 @LiveEvent("intro_message")
@@ -373,6 +398,9 @@ class IntroMessageEvent(AbstractEvent):
     streamer: Optional[User] = None
     """User payload of information about the streamer"""
 
+    language: Optional[str] = None
+    """Language of the creator's room"""
+
 
 @LiveEvent("mic_battle_start")
 class MicBattleStartEvent(AbstractEvent):
@@ -409,7 +437,7 @@ class MicBattleStartEvent(AbstractEvent):
 
         return {'battle_users': battle_users}
 
-    battle_users: List[LinkUser] = field(default_factory=lambda: [])
+    battle_users: List[LinkUser] = field(default_factory=list)
     """Information about the users engaged in the Mic Battle"""
 
 
@@ -447,19 +475,37 @@ class MicBattleUpdateEvent(AbstractEvent):
             for battle_group in battle_item.get('battleGroups', list()):
                 battle_armies.append({
                     'host_user_id': battle_item.get('hostUserId'),
-                    'points': battle_group.get('points'),
-                    'participants': battle_group.get('users')
+                    'points': battle_group.get('points') or None,
+                    'participants': battle_group.get('users') or list()
                 })
 
         event_data['battle_armies'] = battle_armies
         return event_data
 
     battle_status: Optional[int] = None
-    """The status of the current Battle"""
+    """The status of the current Battle. If battle_status=1, the battle is ongoing. If it's 2, the battle has ended."""
 
-    battle_armies: List[BattleArmy] = field(default_factory=lambda: [])
+    battle_armies: List[BattleArmy] = field(default_factory=list)
     """Information about the users engaged in the Mic Battle"""
 
+    @property
+    def in_battle(self) -> bool:
+        """
+        Whether the users are currently in battle
+
+        """
+
+        return self.battle_status == 1
+
+    @property
+    def battle_finished(self) -> bool:
+        """
+        Whether the battle is currently finished
+
+        """
+
+        return self.battle_status == 2
+
 
 @LiveEvent("unknown")
 class UnknownEvent(AbstractEvent):
